<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class ProdutividadeVendedor extends CrudWithParameterBag {
    protected $dataSourceName = 'vnd09900_relProdutividadePorVendedor';
    protected $dataSourceNameFilterAbertura = 'vnd09900_relProdutividadePorVendedor_abertura';
    protected $dataSourceNameFilterProdutoAvo = 'select_produto_avo';
    protected $parameterBagColumns = array('DTMOVTURCAIX', 'CDFILIAL', 'DATAINI', 'DATAFIN', 'CDLOJA', 'CDNVPRODUTO');
    protected $paramBagAberturaCx  = array('NRORG', 'CDFILIAL', 'DATAINI', 'DATAFIN');
    protected $paramBagProdutoAvo  = array('NRORG', 'CDNVPRODUTO');
    protected $ProdutividadeVendedorService;
    protected $utilService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag,  Utilities\Util $utilService,  Services\ProdutividadeVendedor $ProdutividadeVendedorService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->utilService = $utilService;
        $this->ProdutividadeVendedorService = $ProdutividadeVendedorService;
    }

    private function setTCDandCDcondition(&$conditions, $key, $keyName) {
        if( !array_key_exists($key, $conditions) ) {
            $conditions['T'.$keyName] = 'T';    
            $conditions[$keyName] = null;    
        } else {
            $conditions[$keyName] = '_'.implode('_', $conditions[$keyName] ).'_';
            $conditions['T'.$keyName] = null;
            if($key !== $keyName) {
                unset($conditions[$keyName]);
            }
        }
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $result = array();
        try {
            $conditions = $this->utilService->getParamsAsArray($request);
            $conditions['__is_report'] = ($request->getFilterCriteria()->getPage() === null); 
            if( array_key_exists('DTMOVTURCAIX', $conditions) ) {
                $conditions['DATAINI'] = $conditions['DTMOVTURCAIX'][0];
                $conditions['DATAFIN'] = $conditions['DTMOVTURCAIX'][1];
                unset($conditions['DTMOVTURCAIX']);
                if(isset($conditions['DTABERCAIX'])){
                    $conditions['DTMOVTURCAIX'] = $conditions['DTABERCAIX'];
                    unset($conditions['DTABERCAIX']);
                }
            }
            if( !array_key_exists('CDFILIAL', $conditions) ) {
                $conditions['CDFILIAL'] = null;
            }

            if(!isset($conditions['TIPORELATORIO'])){
                $conditions['TIPORELATORIO'] = 'A';
            }
            if( array_key_exists('LABELFILIAL', $conditions) ) {
                $conditions['CDFILIAL'] = $conditions['LABELFILIAL'];
                unset($conditions['LABELFILIAL']);
            }
            if( array_key_exists('LABELLOJA', $conditions) ) {
                $conditions['CDLOJA'] = $conditions['LABELLOJA'];
                unset($conditions['LABELLOJA']);
            }
            if( array_key_exists('GARCON', $conditions) ) {
                $conditions['CDVENDEDOR'] = $conditions['GARCON'];
                unset($conditions['GARCON']);
            }
            if(!is_array($conditions['CDFILIAL'])){
                $filial = array(
                    $conditions['CDFILIAL']
                );
                $conditions['CDFILIAL'] = $filial;
            }
            $this->setTCDandCDcondition($conditions, 'CDFILIAL', 'CDFILIAL');
            $this->setTCDandCDcondition($conditions, 'CDVENDEDOR', 'CDVENDEDOR');
            $this->setTCDandCDcondition($conditions, 'CDLOJA', 'CDLOJA');
            $this->setTCDandCDcondition($conditions, 'DTMOVTURCAIX', 'DTMOVTURCAIX');
            $this->setTCDandCDcondition($conditions, 'CDPRODUTO', 'CDPRODUTO');
            $this->setTCDandCDcondition($conditions, 'CDPRODUTOAVO', 'CDPRODUTOAVO');
            if (isset($conditions['NMPRODUTO'])){
                unset ( $conditions['NMPRODUTO']);
                unset ( $conditions['CDARVPROD']);
            }
            $result = $this->ProdutividadeVendedorService->getProdutividadeVendedor($conditions);            
            $response->addDataSet(new DataSet ('vnd09900_relProdutividadePorVendedor', $result));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function setProducts (){
        
    }

    public function findAbertura(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameFilterAbertura;
        $this->parameterBagColumns = $this->paramBagAberturaCx;
        parent::find($request, $response);
    }

    public function findProdutoAvo(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameFilterProdutoAvo;
        $this->parameterBagColumns = $this->paramBagProdutoAvo;
        parent::find($request, $response);
    }

}