<?php

namespace Teknisa\MAN\Factories;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Geral\Entities\Paramgeral;
use Teknisa\MAN\Utilities;

class CadastramentosAutomaticoFactory {
  
    protected $connection;
    protected $util;

    public function __construct(EntityManager $entityManager, Utilities\Util $util) {
      $this->connection = $entityManager->getConnection();
      $this->entityManager = $entityManager;
      $this->util = $util;
    }
    
    public function selectPrimaryKey($tableName) { 
    
        $sql = "SELECT COLUMN_NAME FROM ALL_CONS_COLUMNS 
                    WHERE TABLE_NAME = :PARAMS
                    AND CONSTRAINT_NAME = 'PK_'||:PARAMS
                    AND COLUMN_NAME <> 'NRORG'";
        
        $std = $this->connection->prepare($sql);
        
        $params = array(
            "PARAMS"       => $tableName
        );
        
        $std->execute($params);
        return $std->fetch();
    }
   
    public function verificaPrimaryKey($pk, $primaryKey, $table) { 
        $sql = 'SELECT * FROM '.$table.' 
                WHERE '.$pk.'=:PARAMS';
        
        $std = $this->connection->prepare($sql);
        
        $params = array(
            "PARAMS"       => $primaryKey
        );
        
        $std->execute($params);
        return $std->fetch();
        
    }
    
    public function verificaDado($row, $primaryKey, $table) { 
        $sql = 'SELECT * FROM '.$table.' 
                WHERE '.$primaryKey.'=:PARAMS';
        
        $std = $this->connection->prepare($sql);
        
        $params = array(
            "PARAMS"       => $row[$primaryKey]
        );
        
        $std->execute($params);
        return $std->fetch();
        
    }
    
    public function verificaFilial($cdfilial) { 
        $sql = 'SELECT CDFILIAL FROM FILIAL WHERE CDFILIAL = :CDFILIAL';
        $std = $this->connection->prepare($sql);
        $params = array(
            "CDFILIAL" => $cdfilial
        );
        $std->execute($params);
        $result = $std->fetch();
        
        if(!empty($result['CDFILIAL'])){
            return true;
        }else{
            return false;
        }
    }
    
    public function verificaLoja($cdfilial, $cdloja) { 
        $sql = 'SELECT CDFILIAL, CDLOJA FROM LOJA WHERE CDFILIAL = :CDFILIAL AND CDLOJA LIKE :CDLOJA';
        $std = $this->connection->prepare($sql);
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDLOJA" => '%'.$cdloja.'%'
        );
        $std->execute($params);
        $result = $std->fetch();
        
        if(!empty($result['CDFILIAL']) && !empty($result['CDLOJA'])){
            return true;
        }else{
            return false;
        }
    }
    
    public function verificaNCaixa($cdfilial, $cdcaixa) { 
        $sql = 'SELECT CDCAIXA,CDFILIAL  FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDCAIXA = :CDCAIXA';
        $std = $this->connection->prepare($sql);
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDCAIXA" => $cdcaixa
        );
        
        $std->execute($params);
        $result = $std->fetch();
        
        if(!empty($result['CDFILIAL']) && !empty($result['CDCAIXA'])){
            return false;
        }else{
            return true;
        }
    }
    
    public function verificaNCaixaExiste($cdfilial, $cdcaixa) { 
        $sql = 'SELECT CDCAIXA,CDFILIAL FROM CAIXA WHERE CDFILIAL = :CDFILIAL AND CDCAIXA = :CDCAIXA';
        $std = $this->connection->prepare($sql);
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDCAIXA" => $cdcaixa
        );
        $std->execute($params);
        $result = $std->fetch();
        
        if(!empty($result['CDFILIAL']) && !empty($result['CDCAIXA'])){
            return true;
        }else{
            return false;
        }
    }
    
    public function verificaNRCONFTELA($nrconftela, $cdfilial, $booleano = true) { 
        $sql = 'SELECT NRCONFTELA, CDFILIAL, DTINIVIGENCIA FROM CONFTELA WHERE NRCONFTELA = :NRCONFTELA AND CDFILIAL = :CDFILIAL';
        $std = $this->connection->prepare($sql);
        $params = array(
            "NRCONFTELA" => $nrconftela,
            "CDFILIAL" => $cdfilial
        );
        
        $std->execute($params);
        $result = $std->fetch();
        if(!empty($result['CDFILIAL']) && !empty($result['NRCONFTELA'])){
            if($booleano){
                return true;
            }
        }else{
             if($booleano){
                return false;
            }
        }
    }
    
    public function verificaSetor($cdsetor) { 
        $sql = 'SELECT CDSETOR FROM SETOR WHERE CDSETOR = :CDSETOR';
        $std = $this->connection->prepare($sql);
        $params = array(
            "CDSETOR" => $cdsetor
        );
        $std->execute($params);
        $result = $std->fetch();
        
        if(!empty($result['CDSETOR'])){
            return true;
        }else{
            return false;
        }
    }
    
    public function verificaTipoCaixa($tipoCaixa) { 
       $tipoCaixa = self::formatValue($tipoCaixa);
       if($tipoCaixa == 'VENDA' || $tipoCaixa == 'CONTROLE' || $tipoCaixa == '1' || $tipoCaixa == '2'){
           return true;
       }else{
           return false;
       }
    }
    
    private function tipoCaixa($value) { 
       $value = self::formatValue($value);
       if($value == 'VENDA'){
           return '1';
       }else if($value == 'CONTROLE'){
           return '2';
       }else{
           return $value;
       }
    }
    
    public function verificaHabilitadoCaixa($tipoCaixa) { 
       $tipoCaixa = self::formatValue($tipoCaixa);
       if($tipoCaixa == 'CONSUMIDOR' || $tipoCaixa == 'CONSUMIDOR/CLIENTE' || $tipoCaixa == 'S' || $tipoCaixa == 'N'){
           return true;
       }else{
           return false;
       }
    }
    
    private function habilitadoCaixa($value) { 
       $value = self::formatValue($value);
       if($value == 'CONSUMIDOR'){
           return 'S';
       }else if($value == 'CONSUMIDOR/CLIENTE'){
           return 'N';
       }else{
           return $value;
       }
    }
    
    public function verificaModalidadeCaixa($modalidadeCaixa) { 
       $columnName = self::formatValue($modalidadeCaixa);
        switch($columnName){
            case $columnName == 'POS' || $columnName == 'VENDA BALCÃO':
                $value = true;
                break;
            case $columnName == 'CMD' || $columnName == 'COMANDA COM VENDA BALCÃO':
                $value = true;
                break;
            case $columnName == 'RES' || $columnName == 'RESTAURANTE COM VENDA BALCÃO':
                $value = true;
                break;
            case $columnName == 'FOS' || $columnName == 'RESTAURANTE/COMANDA/DELIVERY COM VENDA BALCÃO':
                $value = true;
                break;
            case $columnName == 'PKC' || $columnName == 'POCKET COMANDA':
                $value = true;
                break;
            case $columnName == 'PKR' || $columnName == 'POCKET RESTAURANTE':
                $value = true;
                break;
            case $columnName == 'TAA' || $columnName == 'TERMINAL DE AUTO-ATENDIMENTO':
                $value = true;
                break;
            case $columnName == 'EVB' || $columnName == 'DELIVERY VENDA BALCÃO':
                $value = true;
                break;
            case $columnName == 'CTL' || $columnName == 'CONTROLE DE PRODUÇÃO':
                $value = true;
                break;
            case $columnName == 'CSE' || $columnName == 'CONTROLE DE SAÍDA/ENTREGA':
                $value = true;
                break;
            case $columnName == 'FKB' || $columnName == 'MICROTERMINAL':
                $value = true;
                break;
            case $columnName == 'TOT' || $columnName == 'TERMINAL DE CONSULTA':
                $value = true;
                break;
            case $columnName == 'APC' || $columnName == 'AUTO PAGAMENTO COMANDA':
                $value = true;
                break;
            case $columnName == 'CPE' || $columnName == 'CONTROLE DE PEDIDOS':
                $value = true;
                break;
            case $columnName == 'TCO' || $columnName == 'TERMINAL DE CHECKOUT DE COMANDA':
                $value = true;
                break;
            case $columnName == 'CDE' || $columnName == 'CALL CENTER DELIVERY':
                $value = true;
                break;
            case $columnName == 'SDE' || $columnName == 'SITE DELIVERY':
                $value = true;
                break;
            case $columnName == 'CAT' || $columnName == 'CATRACA':
                $value = true;
                break;
            case $columnName == 'APP' || $columnName == 'APLICATIVO MOBILE':
                $value = true;
                break;
            case $columnName == 'OGO' || $columnName == 'ODHENGO - INTELIGÊNCIA ARTIFICIAL':
                $value = true;
                break;
            default: 
                $value = false;
        }
        return $value;
    }
    
    public function verificaTipoButton($tipoButton) { 
       $columnName = self::formatValue($tipoButton);
        switch($columnName){
            case $columnName == '2' || $columnName == 'GRUPO DE PRODUTOS':
                $value = true;
                break;
            case $columnName == '7' || $columnName == 'NAVEGADOR DIREITA':
                $value = true;
                break;
            case $columnName == '8' || $columnName == 'NAVEGADOR ESQUERDA':
                $value = true;
                break;
            default: 
                $value = false;
        }
        return $value;
    }
    
    private function tipoButton($tipoButton) { 
       $columnName = self::formatValue($tipoButton);
        switch($columnName){
            case $columnName == 'GRUPO DE PRODUTOS':
                $value = '2';
                break;
            case $columnName == 'NAVEGADOR DIREITA':
                $value = '7';
                break;
            case $columnName == 'NAVEGADOR ESQUERDA':
                $value = '8';
                break;
            default: 
                $value = $columnName;
        }
        return $value;
    }
    
    public function verificaShowTAA($tipShowTAA) { 
       $columnName = self::formatValue($tipShowTAA);
        switch($columnName){
            case $columnName == 'S' || $columnName == 'SIM':
                $value = true;
                break;
            case $columnName == 'N' || $columnName == 'NÃO' || $columnName == 'NAO':
                $value = true;
                break;
            default: 
                $value = false;
        }
        return $value;
    }
    
    public function verificaTipoSimNao($tipoSimNao) { 
       $columnName = self::formatValue($tipoSimNao);
        switch($columnName){
            case $columnName == 'S' || $columnName == 'SIM':
                $value = true;
                break;
            case $columnName == 'N' || $columnName == 'NÃO' || $columnName == 'NAO':
                $value = true;
                break;
            default: 
                $value = false;
        }
        return $value;
    }
    
    public function tipoSimNao($tipoSimNao) { 
       $columnName = self::formatValue($tipoSimNao);
        switch($columnName){
            case $columnName == 'SIM':
                $value = 'S';
                break;
            case $columnName == 'NÃO' || $columnName == 'NAO':
                $value = 'N';
                break;
            default: 
                $value = $columnName;
        }
        return $value;
    }
    
    private function modalidadeCaixa($value) { 
       $columnName = self::formatValue($value);
        switch($columnName){
            case $columnName == 'VENDA BALCÃO':
                $value = 'POS';
                break;
            case $columnName == 'COMANDA COM VENDA BALCÃO':
                $value = 'CMD';
                break;
            case $columnName == 'RESTAURANTE COM VENDA BALCÃO':
                $value = 'RES';
                break;
            case $columnName == 'RESTAURANTE/COMANDA/DELIVERY COM VENDA BALCÃO':
                $value = 'FOS';
                break;
            case $columnName == 'POCKET COMANDA':
                $value = 'PKC';
                break;
            case $columnName == 'POCKET RESTAURANTE':
                $value = 'PKR';
                break;
            case $columnName == 'TERMINAL DE AUTO-ATENDIMENTO':
                $value = 'TAA';
                break;
            case $columnName == 'DELIVERY VENDA BALCÃO':
                $value = 'EVB';
                break;
            case $columnName == 'CONTROLE DE PRODUÇÃO':
                $value = 'CTL';
                break;
            case $columnName == 'CONTROLE DE SAÍDA/ENTREGA':
                $value = 'CSE';
                break;
            case $columnName == 'MICROTERMINAL':
                $value = 'FKB';
                break;
            case $columnName == 'TERMINAL DE CONSULTA':
                $value = 'TOT';
                break;
            case $columnName == 'AUTO PAGAMENTO COMANDA':
                $value = 'APC';
                break;
            case $columnName == 'CONTROLE DE PEDIDOS':
                $value = 'CPE';
                break;
            case $columnName == 'TERMINAL DE CHECKOUT DE COMANDA':
                $value = 'TCO';
                break;
            case $columnName == 'CALL CENTER DELIVERY':
                $value = 'CDE';
                break;
            case $columnName == 'SITE DELIVERY':
                $value = 'SDE';
                break;
            case $columnName == 'CATRACA':
                $value = 'CAT';
                break;
            case $columnName == 'APLICATIVO MOBILE':
                $value = 'APP';
                break;
            case $columnName == 'ODHENGO - INTELIGÊNCIA ARTIFICIAL':
                $value = 'OGO';
                break;
            default: 
                $value = $columnName;
        }
        return $value;
    }
    
    public function verificaTipoEmissaoCaixa($tipoEmissaoCaixa) { 
       $columnName = self::formatValue($tipoEmissaoCaixa);
        switch($columnName){
            case $columnName == 'ECF' || $columnName == 'CUPOM FISCAL':
                $value = true;
                break;
            case $columnName == 'FNC' || $columnName == 'NFC-E CONSUMIDOR':
                $value = true;
                break;
            case $columnName == 'SAT' || $columnName == 'CF-E-SAT':
                $value = true;
                break;
            default: 
                $value = false;
        }
        return $value;
    }
    
    private function tipoEmissaoCaixa($tipoEmissaoCaixa) { 
       $columnName = self::formatValue($tipoEmissaoCaixa);
        switch($columnName){
            case $columnName == 'CUPOM FISCAL':
                $value = 'ECF';
                break;
            case $columnName == 'NFC-E CONSUMIDOR':
                $value = 'FNC';
                break;
            case $columnName == 'CF-E-SAT':
                $value = 'SAT';
                break;
            default: 
                $value = $columnName;
        }
        return $value;
    }
    
    private function formatValue($value){
        $value = trim($value);
        $value = mb_strtoupper($value, 'UTF-8');
        
        return $value;
    }
    
    public function insertGrupOperadores($data){
        $posicoes = array_fill(0, 100, false);
        $return = array(
            "operacao" => true,
            "error" => ''
        );
        try{
            $this->entityManager->beginTransaction();
            foreach($data as $key => $value){
                $CDGRUPOPER = $this->newCodigoTabela('GRUPOPER', 2, null, $posicoes);
                if(!empty($CDGRUPOPER)){
                    $sql = "INSERT INTO GRUPOPER (CDGRUPOPER,DSGRUPOPER,IDUTCTRSENOP,NRORG)
                        VALUES (:CDGRUPOPER,
                                :DSGRUPOPER,
                                :IDUTCTRSENOP,
                                :NRORG)";
                
                    $params = array(
                        'CDGRUPOPER'=> $CDGRUPOPER,
                        'DSGRUPOPER'=>iconv("UTF-8", "ISO-8859-1", $value['DSGRUPOPER']), 
                        'IDUTCTRSENOP'=>self::tipoSimNao($value['IDUTCTRSENOP']),
                        'NRORG'=> $value['NRORG']
                    );
                    $std = $this->connection->prepare($sql);
                    $result = $std->execute($params);
                }else{
                    $linha = $key + 2 ;
                    $errors[][$linha] = 'Já não há códigos válidos para cadastrar grupo de operadores! Linha '. $linha. ' Grupo de Operadores ' .  $value['DSGRUPOPER'];
                }
            }
            $this->entityManager->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            $return['operacao'] = false;
            $return['error'] = $e->getMessage();
            //throw $e;
        }
        
        if(!empty($errors) && count($errors) > 0){
            $return['error'] = $errors;
        }
        
        return $return;   
    }
    
    public function insertGrupoPromocional($data){
        $posicoes = array_fill(0, 100000, false);
        $return = array(
            "operacao" => true,
            "error" => ''
        );
        try{
            $this->entityManager->beginTransaction();
            foreach($data as $key => $value){
                $CDGRUPROMOC = $this->newCodigoTabela('GRUPROMOC', 10, null, $posicoes);
                if(!empty($CDGRUPROMOC)){
                    $sql = "INSERT INTO GRUPROMOC (CDGRUPROMOC,NMGRUPROMOC,NRORG)
                            VALUES (:CDGRUPROMOC,
                                    :NMGRUPROMOC,
                                    :NRORG)";
                
                    $params = array(
                        'CDGRUPROMOC'=> $CDGRUPROMOC,
                        'NMGRUPROMOC'=>iconv("UTF-8", "ISO-8859-1", $value['NMGRUPROMOC']),
                        'NRORG'=> $value['NRORG']
                    );
                    
                    $std = $this->connection->prepare($sql);
                    $result = $std->execute($params);
                }else{
                    $linha = $key + 2 ;
                    $errors[][$linha] = 'Já não há códigos válidos para cadastrar grupos promocionais! Linha '. $linha. ' Grupo Promocional ' .  $value['NMGRUPROMOC'];
                }
            }
            $this->entityManager->commit();
        }catch (\Exception $e) {
            //throw $e;
            $this->entityManager->rollback();
            $return['operacao'] = false;
            $return['error'] = $e->getMessage();
        }
        
        if(!empty($errors) && count($errors) > 0){
            $return['error'] = $errors;
        }
        
        return $return;   
    }
    
    public function insertGrupoObservacao($data){
        $posicoes = array_fill(0, 100, false);
        $return = array(
            "operacao" => true,
            "error" => ''
        );
        try{
            $this->entityManager->beginTransaction();
            foreach($data as $key => $value){
                $CDGRPOCOR = $this->newCodigoTabela('GRUPOCOR', 2, null, $posicoes);
                if(!empty($CDGRPOCOR)){
                    $sql = "INSERT INTO GRUPOCOR (CDGRPOCOR,DSGRPOCOR,NRORG, IDTIPGRP)
                            VALUES (:CDGRPOCOR,
                                    :DSGRPOCOR,
                                    :NRORG,
                                    :IDTIPGRP)";
                
                    $params = array(
                        'CDGRPOCOR'=> $CDGRPOCOR,
                        'DSGRPOCOR'=>iconv("UTF-8", "ISO-8859-1", $value['DSGRPOCOR']), 
                        'NRORG'=> $value['NRORG'],
                        'IDTIPGRP' => 3
                    );
                    
                    $std = $this->connection->prepare($sql);
                    $result = $std->execute($params);
                }else{
                    $linha = $key + 2 ;
                    $errors[][$linha] = 'Já não há códigos válidos para cadastrar grupos de observações! Linha '. $linha. ' Grupo de Observação ' .  $value['DSGRPOCOR'];
                }
            }
            $this->entityManager->commit();
        }catch (\Exception $e) {
            //throw $e;
            $this->entityManager->rollback();
            $return['operacao'] = false;
            $return['error'] = $e->getMessage();
        }
        
        if(!empty($errors) && count($errors) > 0){
            $return['error'] = $errors;
        }
        
        return $return;  
    }
    
    public function insertMesa($data){
        $return = array(
            "operacao" => true,
            "error" => ''
        );
        try{
            $this->entityManager->beginTransaction();
            foreach($data as $key => $value){
                $CDSALA = $this->newCodigoTabela('SALA', 5, $value['CDFILIAL']);
                if(!empty($CDSALA)){
                    $sql = "INSERT INTO SALA (CDFILIAL, CDSALA, SGSALA, NMSALA, NRVAGASSALA, NRORG)
                            VALUES (:CDFILIAL,
                                    :CDSALA,
                                    :SGSALA,
                                    :NMSALA,
                                    :NRVAGASSALA,
                                    :NRORG)";
                
                    $params = array(
                        'CDFILIAL' => $value['CDFILIAL'],
                        'CDSALA'=> $CDSALA,
                        'SGSALA'=>$value['SGSALA'],
                        'NMSALA'=>iconv("UTF-8", "ISO-8859-1", $value['NMSALA']),
                        'NRVAGASSALA'=>self::leftZeroFill($value['NRVAGASSALA'], 3),
                        'NRORG'=> $value['NRORG']
                    );
                    
                    $std = $this->connection->prepare($sql);
                    $result = $std->execute($params);
                }else{
                    $linha = $key + 2 ;
                    $errors[][$linha] = 'Já não há códigos válidos para cadastrar ambiente! Linha '. $linha. ' Ambiente ' .  $value['NMSALA'];
                }
            }
            $this->entityManager->commit();
        }catch (\Exception $e) {
            //throw $e;
            $this->entityManager->rollback();
            $return['operacao'] = false;
            $return['error'] = $e->getMessage();
        }
        return $return; 
    }
    
    public function insertCaixa($data){
        $posicoes = array_fill(0, 1000, false);
        $return = array(
            "operacao" => true,
            "error" => ''
        );
        try{
            $this->entityManager->beginTransaction();
            foreach($data as $key => $value){
                $CDCAIXA = $this->newCodigoTabela('CAIXA', 3, $value['CDFILIAL'], $posicoes);
                if(!empty($CDCAIXA)){
                    $sql = "INSERT INTO CAIXA (CDFILIAL, CDLOJA, CDCAIXA, NMCAIXA, IDCONFVENDA, IDCAIXAFISCA, IDHABCAIXAVENDA, IDTPEMISSAOFOS, CDCAIXACONT, CDSETOR, IDUTCXDRIVETHU, NRORG)
                            VALUES (:CDFILIAL,
                                    :CDLOJA,
                                    :CDCAIXA,
                                    :NMCAIXA,
                                    :IDCONFVENDA,
                                    :IDCAIXAFISCA,
                                    :IDHABCAIXAVENDA,
                                    :IDTPEMISSAOFOS,
                                    :CDCAIXACONT,
                                    :CDSETOR,
                                    :IDUTCXDRIVETHU,
                                    :NRORG)";
                
                    $params = array(
                        'CDFILIAL' => $value['CDFILIAL'],
                        'CDLOJA' => $value['CDLOJA'],
                        'CDCAIXA' => $CDCAIXA,
                        'NMCAIXA'=>iconv("UTF-8", "ISO-8859-1", $value['NMCAIXA']),
                        'IDCONFVENDA'=>self::tipoCaixa($value['IDCONFVENDA']),
                        'IDCAIXAFISCA'=>self::habilitadoCaixa($value['IDCAIXAFISCA']),
                        'IDHABCAIXAVENDA'=>self::modalidadeCaixa($value['IDHABCAIXAVENDA']),
                        'IDTPEMISSAOFOS'=>self::tipoEmissaoCaixa($value['IDTPEMISSAOFOS']),
                        'CDCAIXACONT'=>$value['CDCAIXACONT'],
                        'CDSETOR'=>$value['CDSETOR'],
                        'IDUTCXDRIVETHU'=> 'S',
                        'NRORG'=> $value['NRORG']
                    );
                    $std = $this->connection->prepare($sql);
                    $result = $std->execute($params);
                }else{
                    $linha = $key + 2 ;
                    $errors[][$linha] = 'Já não há códigos válidos para cadastrar caixas! Linha '. $linha. ' Caixa ' .  $value['NMCAIXA'];
                }
            }
            
            $this->entityManager->commit();
        }catch (\Exception $e) {
            //throw $e;
            $this->entityManager->rollback();
            $return['operacao'] = false;
            $return['error'] = $e->getMessage();
        }
        
        if(!empty($errors) && count($errors) > 0){
            $return['error'] = $errors;
        }
        
        return $return; 
    }
    
    public function insertConfiguracaoGpProdutos($data){
        $return = array(
            "operacao" => true,
            "error" => ''
        );
        
        $errors = array();
        
        try{
            $this->entityManager->beginTransaction();
            foreach($data as $key => $value){
                if(self::verficaPosition($value, "GP")){
        
                    $resultVigencia = $this->retornaVigenciaAtiva($value);
                    $sql = "INSERT INTO ITMENUCONFTE (CDFILIAL, NRCONFTELA, NRPGCONFTELA, 
                            NRBUTTON, DSBUTTON, IDTPBUTTON, DSBUTTONESPANH, 
                            DSBUTTONINGLES, NRCOLORBACK, NRCOLORTEXT, IDSHOWTAA, NRORG, NRFONTETEXT,NRPGCONFTAUX, NRBUTTONAUX, IDTPBUTONAUX, DTINIVIGENCIA)
                                    VALUES (:CDFILIAL,
                                            :NRCONFTELA,
                                            :NRPGCONFTELA,
                                            :NRBUTTON,
                                            :DSBUTTON,
                                            :IDTPBUTTON,
                                            :DSBUTTONESPANH,
                                            :DSBUTTONINGLES,
                                            :NRCOLORBACK,
                                            :NRCOLORTEXT,
                                            :IDSHOWTAA,
                                            :NRORG,
                                            :NRFONTETEXT,
                                            :NRPGCONFTAUX,
                                            :NRBUTTONAUX,
                                            :IDTPBUTONAUX,
                                            :DTINIVIGENCIA)";
                        
                            $params = array(
                                'CDFILIAL' => $value['CDFILIAL'],
                                'NRCONFTELA' => $value['NRCONFTELA'],
                                'NRPGCONFTELA' => $value['NRPGCONFTELA'],
                                'NRBUTTON' => $value['NRBUTTON'],
                                'DSBUTTON' => $value['DSBUTTON'],
                                'IDTPBUTTON' => self::tipoButton($value['IDTPBUTTON']),
                                'DSBUTTONESPANH' => $value['DSBUTTONESPANH'],
                                'DSBUTTONINGLES' => $value['DSBUTTONINGLES'],
                                'NRCOLORBACK' => $value['NRCOLORBACK'],
                                'NRCOLORTEXT' => $value['NRCOLORTEXT'],
                                'IDSHOWTAA' => self::tipoSimNao($value['IDSHOWTAA']),
                                'NRORG'=> $value['NRORG'],
                                'NRFONTETEXT'=> !empty($value['NRFONTETEXT']) ? $value['NRFONTETEXT'] : "08",
                                'NRPGCONFTAUX'=> '0',
                                'NRBUTTONAUX'=> '0',
                                'IDTPBUTONAUX'=> '0',
                                'DTINIVIGENCIA' => $resultVigencia['DTINIVIGENCIA']
                            );
                            $std = $this->connection->prepare($sql);
                            $result = $std->execute($params);
                }else{
                    $linha = $key + 2 ;
                    $errors[][$linha] = 'Já há cadastro na posição informada! Linha '. $linha. ' página ' .  $value['NRPGCONFTELA'] . ' e posição '. $value['NRBUTTON'];
                }
            }
            $this->entityManager->commit();
        }catch (\Exception $e) {
            //throw $e;
            $this->entityManager->rollback();
            $return['operacao'] = false;
            $return['error'] = $e->getMessage();
        }
        
        if(!empty($errors) && count($errors) > 0){
            $return['error'] = $errors;
        }
        return $return; 
    }
    
    public function insertConfiguracaoGpRecebimentos($data){
        $return = array(
            "operacao" => true,
            "error" => ''
        );
        
        $errors = array();
        
        try{
            $this->entityManager->beginTransaction();
            foreach($data as $key => $value){
                $resultVigencia = $this->retornaVigenciaAtiva($value);
                if(self::verficaPosition($value, "GR")){
                    $sql = "INSERT INTO ITMENUCONFTE (CDFILIAL, NRCONFTELA, NRPGCONFTELA, 
                            NRBUTTON, DSBUTTON, IDTPBUTTON, NRCOLORBACK, NRCOLORTEXT, NRORG, NRFONTETEXT, NRPGCONFTAUX, NRBUTTONAUX, IDTPBUTONAUX, DTINIVIGENCIA)
                                    VALUES (:CDFILIAL,
                                            :NRCONFTELA,
                                            :NRPGCONFTELA,
                                            :NRBUTTON,
                                            :DSBUTTON,
                                            :IDTPBUTTON,
                                            :NRCOLORBACK,
                                            :NRCOLORTEXT,
                                            :NRORG,
                                            :NRFONTETEXT,
                                            :NRPGCONFTAUX,
                                            :NRBUTTONAUX,
                                            :IDTPBUTONAUX,
                                            :DTINIVIGENCIA)";
                        
                            $params = array(
                                'CDFILIAL' => $value['CDFILIAL'],
                                'NRCONFTELA' => $value['NRCONFTELA'],
                                'NRPGCONFTELA' => 1,
                                'NRBUTTON' => $value['NRBUTTON'],
                                'DSBUTTON' => $value['DSBUTTON'],
                                'IDTPBUTTON' => 6,
                                'NRCOLORBACK' => $value['NRCOLORBACK'],
                                'NRCOLORTEXT' => $value['NRCOLORTEXT'],
                                'NRORG'=> $value['NRORG'],
                                'NRFONTETEXT'=> !empty($value['NRFONTETEXT']) ? $value['NRFONTETEXT'] : "08",
                                'NRPGCONFTAUX'=> '0',
                                'NRBUTTONAUX'=> '0',
                                'IDTPBUTONAUX'=> '0',
                                'DTINIVIGENCIA'=> $resultVigencia['DTINIVIGENCIA']
                            );
                            $std = $this->connection->prepare($sql);
                            $result = $std->execute($params);
                }else{
                    $linha = $key + 2 ;
                    $errors[][$linha] = 'Já há cadastro na posição informada! Linha '. $linha. ' posição '. $value['NRBUTTON'];
                }
            }
            $this->entityManager->commit();
        }catch (\Exception $e) {
            //throw $e;
            $this->entityManager->rollback();
            $return['operacao'] = false;
            $return['error'] = $e->getMessage();
        }
        
        
        if(!empty($errors) && count($errors) > 0){
            $return['error'] = $errors;
        }
        
        return $return; 
    }
    
    public function retornaVigenciaAtiva($value){
        $sqlVigencia = 'SELECT DTINIVIGENCIA FROM CONFTELA WHERE NRCONFTELA = :NRCONFTELA AND CDFILIAL = :CDFILIAL';
        $std = $this->connection->prepare($sqlVigencia);
        $params = array(
            "NRCONFTELA" => $value['NRCONFTELA'],
            "CDFILIAL" => $value['CDFILIAL']
        );
        $std->execute($params);
        return $std->fetch();
    }
    
    private function verficaPosition($value, $tipo){
        
        $resultVigencia = $this->retornaVigenciaAtiva($value);
        if($tipo == "GP"){
            $tipoButton = '2_7_8';
        }
        
        if($tipo == "GR"){
            $tipoButton = '6';
        }
        
        $sql = 'SELECT NRBUTTON 
        FROM ITMENUCONFTE 
        WHERE CDFILIAL = :CDFILIAL 
        AND NRCONFTELA = :NRCONFTELA 
        AND NRPGCONFTELA = :NRPGCONFTELA
        AND NRBUTTON = : NRBUTTON
        AND (INSTR(:IDTPBUTTON, IDTPBUTTON) <> 0)
        AND NRPGCONFTAUX = :NRPGCONFTAUX
        AND NRBUTTONAUX = :NRBUTTONAUX
        AND DTINIVIGENCIA = :DTINIVIGENCIA
        AND IDTPBUTONAUX = :IDTPBUTONAUX';
        
        $std = $this->connection->prepare($sql);
        $params = array(
            "CDFILIAL" => $value['CDFILIAL'],
            "NRCONFTELA" => $value['NRCONFTELA'],
            "NRPGCONFTELA" => isset($value['NRPGCONFTELA']) ? $value['NRPGCONFTELA'] : 1,
            "NRBUTTON" => $value['NRBUTTON'],
            "IDTPBUTTON" => $tipoButton,
            'NRPGCONFTAUX'=> '0',
            'NRBUTTONAUX'=> '0',
            'IDTPBUTONAUX'=> '0',
            'DTINIVIGENCIA' => $resultVigencia['DTINIVIGENCIA']
        );
        
        $std->execute($params);
        $result = $std->fetch();
        
        if(!empty($result['NRBUTTON'])){
            return false;
        }else{
            return true;
        }
    }
    
    public function newCodigoTabela($table, $tamanho = null, $cdfilial = null, $posicoes = array()){
        
        if($table == 'GRUPOPER'){
            $sql = 'SELECT DISTINCT(CDGRUPOPER) FROM GRUPOPER ORDER BY CDGRUPOPER';
            $std = $this->connection->prepare($sql);
            $std->execute();
            $result = $std->fetchAll();
            if(!empty($result)){
                foreach ($result as $key => $value) {
                    if (array_key_exists((int)$value['CDGRUPOPER'], $posicoes)) {
                        $posicoes[(int)$value['CDGRUPOPER']] =  $value['CDGRUPOPER'];
                    }
                }
            }
            foreach($posicoes as $key => $value){
                if(!$value){
                    return self::leftZeroFill($key, $tamanho);
                }
            }
            return "";
        }
        
        if($table == 'GRUPOCOR'){
            
            $sql = 'SELECT DISTINCT(CDGRPOCOR) FROM GRUPOCOR ORDER BY CDGRPOCOR';
            $std = $this->connection->prepare($sql);
            $std->execute();
            $result = $std->fetchAll();
            if(!empty($result)){
                foreach ($result as $key => $value) {
                    if (array_key_exists((int)$value['CDGRPOCOR'], $posicoes)) {
                        $posicoes[(int)$value['CDGRPOCOR']] =  $value['CDGRPOCOR'];
                    }
                }
            }
            foreach($posicoes as $key => $value){
                if(!$value){
                    return self::leftZeroFill($key, $tamanho);
                }
            }
            return "";
        }
        
        if($table == 'GRUPROMOC'){
            
            $sql = 'SELECT DISTINCT(CDGRUPROMOC) FROM GRUPROMOC ORDER BY CDGRUPROMOC';
            $std = $this->connection->prepare($sql);
            $std->execute();
            $result = $std->fetchAll();
            
            if(!empty($result)){
                foreach ($result as $key => $value) {
                    if (array_key_exists((int)$value['CDGRUPROMOC'], $posicoes)) {
                        $posicoes[(int)$value['CDGRUPROMOC']] =  $value['CDGRUPROMOC'];
                    }
                }
            }
            foreach($posicoes as $key => $value){
                if(!$value){
                    return self::leftZeroFill($key, $tamanho);
                }
            }
            return "";
        }
        
        if($table == 'SALA'){
            $params = array(
                'CDFILIAL' => $cdfilial
            );
            $sql = 'SELECT MAX(CDSALA) AS MAX_CDSALA 
                    FROM SALA
                    WHERE CDFILIAL= :CDFILIAL';
                    
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetch();
            
            $codigo = $result['MAX_CDSALA'];
            return self::leftZeroFill(!is_null($codigo) ? ++$codigo : 1, $tamanho);
            
        }
        
        if($table == 'CAIXA'){
            $params = array(
                'CDFILIAL' => $cdfilial
            );
            $sql = 'SELECT DISTINCT(CDCAIXA) FROM CAIXA WHERE CDFILIAL = :CDFILIAL ORDER BY CDCAIXA';
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            if(!empty($result)){
                foreach ($result as $key => $value) {
                    if (array_key_exists((int)$value['CDCAIXA'], $posicoes)) {
                        $posicoes[(int)$value['CDCAIXA']] =  $value['CDCAIXA'];
                    }
                }
            }
            foreach($posicoes as $key => $value){
                if(empty($value) || $value == '' || $value == null){
                     return self::leftZeroFill($key, $tamanho);
                }
            }
            return "";
        }
    }
    
    private function leftZeroFill($string, $length){
        $string = str_pad(strval($string), $length, '0', STR_PAD_LEFT); 
        return $string;
    }
    
  
}