<?php
namespace Teknisa\MAN\Services;

use Doctrine\DBAL\Connection;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services\NovoCodigo;

class AprovacaoPrevisaoEstoqueService {
    
    protected $conn;

    public function __construct(Connection $conn) {
        $this->conn = $conn;
    }

    
    public function getSenhaOperador($rows){
         $params = array (
            ':CDOPERADOR'     => $rows['CDOPERADOR'],
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->fetchAssoc(self::GET_SENHA_OPER,$params)['CDSENHAOPERWEB'];
    }
    
    public function adqPrevisaoPDVLim($rows){
        $params = array (
            ':NRSEQPREVPDV' => $rows['NRSEQPREVPDV'],
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->fetchAssoc(self::ADQ_PREVISAOPDV_LIM,$params)['DTPREVCONPDV'];
    }
    
    public function adqCalePrevConsPDV($rows, $adqPrevisaoPDVLim){
        $params = array (
            ':DTPREVCONPDV' => $adqPrevisaoPDVLim,
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->fetchAssoc(self::ADQ_CALEPREVCONS_PDV,$params);
    }
    
    private function returnParams ($rows, $produto, $IDAPROVPREVCONS){
        return array (
            ':CDPRODUTO'        => $produto['CDPRODUTO'],
            ':NRPRODCONPDV'     => $produto['NRPRODCONPDV'],
            ':CDOPERADOR'       => $rows['CDOPERADOR'],
            ':CDFILIAL'         => $produto['CDFILIAL'],
            ':NRORG'            => $rows['NRORG'],
            ':IDAPROVPREVCONS'  => $IDAPROVPREVCONS,
            ':NRSEQPREVPDV'     => $rows['NRSEQPREVPDV']
        );
    }
    
    public function acmPrevConsPDV($rows, $produto, $IDAPROVPREVCONS){
        $params = self::returnParams($rows, $produto, $IDAPROVPREVCONS);
        
        return $this->conn->executeQuery(self::ACM_PREVCONS_PDV,$params);
    }
    
    public function writeLog($rows, $produto, $IDAPROVPREVCONS){
        
        $params = self::returnParams($rows, $produto, $IDAPROVPREVCONS);
        $params['IDLOGPREVCONS'] = ($IDAPROVPREVCONS == 'S') ? 'A' : 'C';
        return $this->conn->executeQuery(self::LOG_PREVISAO,$params);
        
    }
    
    public function updateLog($row){
        $params = array (
            ':CDPRODUTO'        => $row['CDPRODUTO'],
            ':DSLOGPREVCONS'    => $row['DSLOGPREVCONS'],
            ':CDFILIAL'         => $row['CDFILIAL'],
            ':NRORG'            => $row['NRORG'],
            ':NRSEQPREVPDV'     => $row['NRSEQPREVPDV']
        );
        return $this->conn->executeQuery(self::SAVE_DS_LOG,$params);
    }
    
    
    const ACM_PREVCONS_PDV ="
    UPDATE PREVCONSPDV
        SET
          IDAPROVPREVCONS = :IDAPROVPREVCONS
        WHERE
          CDFILIAL = :CDFILIAL AND
          NRSEQPREVPDV = :NRSEQPREVPDV AND
          CDPRODUTO = :CDPRODUTO AND
          NRPRODCONPDV = :NRPRODCONPDV AND
          NRORG = :NRORG
    ";
    
    const ADQ_CALEPREVCONS_PDV = "
    SELECT
      MIN(DTLIMAPROCONPDV) AS DTLIMAPROCONPDV, TO_CHAR(SYSDATE, 'DD/MM/YYYY') AS DTATUAL
    FROM
      CALEPREVCONSPDV
    WHERE DTLIMCALCCONPDV >= :DTPREVCONPDV
    AND NRORG = :NRORG
    ";


   
    const GET_SENHA_OPER = "
    SELECT CDSENHAOPERWEB FROM OPERADOR
        WHERE CDOPERADOR = :CDOPERADOR
        AND NRORG = :NRORG
    ";
    
    const  ADQ_PREVISAOPDV_LIM = "
    SELECT
      TRUNC (DTPREVCONPDV) AS DTPREVCONPDV
    FROM
      PREVISAOPDV
    WHERE CDFILIAL = :CDFILIAL
      AND NRSEQPREVPDV = :NRSEQPREVPDV
      AND NRORG = :NRORG
      ";
      
    const LOG_PREVISAO =  "
    INSERT INTO LOGPREVCONSPDV  (CDFILIAL, NRSEQPREVPDV, CDPRODUTO,
        NRPRODCONPDV, DTLOGPREVCONS, IDLOGPREVCONS, DSLOGPREVCONS, CDOPERADOR)
        VALUES   (:CDFILIAL, :NRSEQPREVPDV, :CDPRODUTO, :NRPRODCONPDV,
        SYSDATE, :IDLOGPREVCONS, '', :CDOPERADOR)";
        
    const SAVE_DS_LOG = "
    UPDATE LOGPREVCONSPDV SET DSLOGPREVCONS = :DSLOGPREVCONS
        WHERE CDFILIAL = :CDFILIAL
        AND NRSEQPREVPDV = :NRSEQPREVPDV
        AND CDPRODUTO = :CDPRODUTO
        AND NRORG = :NRORG";
}
